@@ -7,21 +7,49 @@ import time
 from pathlib import Path
 from shutil import which
 from tempfile import TemporaryDirectory
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Set, Tuple
 
 import userpath  # type: ignore
+from packaging.utils import canonicalize_name
 
 from pipx import constants
 from pipx.colors import bold, red
 from pipx.emojies import hazard, stars
 from pipx.package_specifier import parse_specifier_for_install, valid_pypi_name
 from pipx.pipx_metadata_file import PackageInfo
-from pipx.util import WINDOWS, PipxError, mkdir, rmdir
+from pipx.util import WINDOWS, PipxError, mkdir, pipx_wrap, rmdir
 from pipx.venv import Venv
 
+logger = logging.getLogger(__name__)
+
+
+class VenvProblems:
+    def __init__(
+        self,
+        bad_venv_name: bool = False,
+        invalid_interpreter: bool = False,
+        missing_metadata: bool = False,
+        not_installed: bool = False,
+    ) -> None:
+        self.bad_venv_name = bad_venv_name
+        self.invalid_interpreter = invalid_interpreter
+        self.missing_metadata = missing_metadata
+        self.not_installed = not_installed
+
+    def any_(self) -> bool:
+        return any(self.__dict__.values())
+
+    def or_(self, venv_problems: "VenvProblems") -> None:
+        for attribute in self.__dict__:
+            setattr(
+                self,
+                attribute,
+                getattr(self, attribute) or getattr(venv_problems, attribute),
+            )
+
 
 def expose_apps_globally(
-    local_bin_dir: Path, app_paths: List[Path], *, force: bool, suffix: str = "",
+    local_bin_dir: Path, app_paths: List[Path], *, force: bool, suffix: str = ""
 ) -> None:
     if not _can_symlink(local_bin_dir):
         _copy_package_apps(local_bin_dir, app_paths, suffix=suffix)
@@ -54,7 +82,7 @@ def _can_symlink(local_bin_dir: Path) -> bool:
 
 
 def _copy_package_apps(
-    local_bin_dir: Path, app_paths: List[Path], suffix: str = "",
+    local_bin_dir: Path, app_paths: List[Path], suffix: str = ""
 ) -> None:
     for src_unresolved in app_paths:
         src = src_unresolved.resolve()
@@ -63,14 +91,14 @@ def _copy_package_apps(
         if not dest.parent.is_dir():
             mkdir(dest.parent)
         if dest.exists():
-            logging.warning(f"{hazard}  Overwriting file {str(dest)} with {str(src)}")
+            logger.warning(f"{hazard}  Overwriting file {str(dest)} with {str(src)}")
             dest.unlink()
         if src.exists():
             shutil.copy(src, dest)
 
 
 def _symlink_package_apps(
-    local_bin_dir: Path, app_paths: List[Path], *, force: bool, suffix: str = "",
+    local_bin_dir: Path, app_paths: List[Path], *, force: bool, suffix: str = ""
 ) -> None:
     for app_path in app_paths:
         app_name = app_path.name
@@ -80,7 +108,7 @@ def _symlink_package_apps(
             mkdir(symlink_path.parent)
 
         if force:
-            logging.info(f"Force is true. Removing {str(symlink_path)}.")
+            logger.info(f"Force is true. Removing {str(symlink_path)}.")
             try:
                 symlink_path.unlink()
             except FileNotFoundError:
@@ -92,15 +120,21 @@ def _symlink_package_apps(
         is_symlink = symlink_path.is_symlink()
         if exists:
             if symlink_path.samefile(app_path):
-                logging.info(f"Same path {str(symlink_path)} and {str(app_path)}")
+                logger.info(f"Same path {str(symlink_path)} and {str(app_path)}")
             else:
-                logging.warning(
-                    f"{hazard}  File exists at {str(symlink_path)} and points "
-                    f"to {symlink_path.resolve()}, not {str(app_path)}. Not modifying."
+                logger.warning(
+                    pipx_wrap(
+                        f"""
+                        {hazard}  File exists at {str(symlink_path)} and points
+                        to {symlink_path.resolve()}, not {str(app_path)}. Not
+                        modifying.
+                        """,
+                        subsequent_indent=" " * 4,
+                    )
                 )
             continue
         if is_symlink and not exists:
-            logging.info(
+            logger.info(
                 f"Removing existing symlink {str(symlink_path)} since it "
                 "pointed non-existent location"
             )
@@ -110,9 +144,14 @@ def _symlink_package_apps(
         symlink_path.symlink_to(app_path)
 
         if existing_executable_on_path:
-            logging.warning(
-                f"{hazard}  Note: {app_name_suffixed} was already on your PATH at "
-                f"{existing_executable_on_path}"
+            logger.warning(
+                pipx_wrap(
+                    f"""
+                    {hazard}  Note: {app_name_suffixed} was already on your
+                    PATH at {existing_executable_on_path}
+                    """,
+                    subsequent_indent=" " * 4,
+                )
             )
 
 
@@ -122,7 +161,7 @@ def get_package_summary(
     package: str = None,
     new_install: bool = False,
     include_injected: bool = False,
-) -> str:
+) -> Tuple[str, VenvProblems]:
     venv = Venv(venv_dir)
     python_path = venv.python_path.resolve()
 
@@ -130,19 +169,29 @@ def get_package_summary(
         package = venv.main_package_name
 
     if not python_path.is_file():
-        return f"   package {red(bold(venv_dir.name))} has invalid interpreter {str(python_path)}"
+        return (
+            f"   package {red(bold(venv_dir.name))} has invalid interpreter {str(python_path)}",
+            VenvProblems(invalid_interpreter=True),
+        )
     if not venv.package_metadata:
         return (
-            f"   package {red(bold(venv_dir.name))} has missing internal pipx metadata.\n"
-            f"       It was likely installed using a pipx version before 0.15.0.0.\n"
-            f"       Please uninstall and install this package, or reinstall-all to fix."
+            f"   package {red(bold(venv_dir.name))} has missing internal pipx metadata.",
+            VenvProblems(missing_metadata=True),
+        )
+    if venv_dir.name != canonicalize_name(venv_dir.name):
+        return (
+            f"   package {red(bold(venv_dir.name))} needs its internal data updated.",
+            VenvProblems(bad_venv_name=True),
         )
 
     package_metadata = venv.package_metadata[package]
 
     if package_metadata.package_version is None:
-        not_installed = red("is not installed")
-        return f"   package {bold(package)} {not_installed} in the venv {venv_dir.name}"
+        return (
+            f"   package {red(bold(package))} {red('is not installed')} "
+            f"in the venv {venv_dir.name}",
+            VenvProblems(not_installed=True),
+        )
 
     apps = package_metadata.apps + package_metadata.apps_of_dependencies
     exposed_app_paths = _get_exposed_app_paths_for_package(
@@ -160,22 +209,25 @@ def get_package_summary(
         if venv.pipx_metadata.python_version is not None
         else ""
     )
-    return _get_list_output(
-        python_version,
-        python_path,
-        package_metadata.package_version,
-        package,
-        new_install,
-        exposed_binary_names,
-        unavailable_binary_names,
-        venv.pipx_metadata.injected_packages if include_injected else None,
-        suffix=package_metadata.suffix,
+    return (
+        _get_list_output(
+            python_version,
+            python_path,
+            package_metadata.package_version,
+            package,
+            new_install,
+            exposed_binary_names,
+            unavailable_binary_names,
+            venv.pipx_metadata.injected_packages if include_injected else None,
+            suffix=package_metadata.suffix,
+        ),
+        VenvProblems(),
     )
 
 
 def _get_exposed_app_paths_for_package(
     venv_bin_path: Path, package_binary_names: List[str], local_bin_dir: Path
-):
+) -> Set[Path]:
     bin_symlinks = set()
     for b in local_bin_dir.iterdir():
         try:
@@ -244,8 +296,8 @@ def package_name_from_spec(
         # NOTE: if pypi name and installed package name differ, this means pipx
         #       will use the pypi name
         package_name = pypi_name
-        logging.info(f"Determined package name: {package_name}")
-        logging.info(f"Package name determined in {time.time()-start_time:.1f}s")
+        logger.info(f"Determined package name: {package_name}")
+        logger.info(f"Package name determined in {time.time()-start_time:.1f}s")
         return package_name
 
     # check syntax and clean up spec and pip_args
@@ -258,7 +310,7 @@ def package_name_from_spec(
             package_or_url=package_spec, pip_args=pip_args
         )
 
-    logging.info(f"Package name determined in {time.time()-start_time:.1f}s")
+    logger.info(f"Package name determined in {time.time()-start_time:.1f}s")
     return package_name
 
 
@@ -270,54 +322,44 @@ def run_post_install_actions(
     include_dependencies: bool,
     *,
     force: bool,
-):
+) -> None:
     package_metadata = venv.package_metadata[package]
 
     display_name = f"{package}{package_metadata.suffix}"
 
-    if not package_metadata.app_paths and not include_dependencies:
-        # No apps associated with this package and we aren't including dependencies.
-        # This package has nothing for pipx to use, so this is an error.
-        for dep, dependent_apps in package_metadata.app_paths_of_dependencies.items():
-            print(
-                f"Note: Dependent package '{dep}' contains {len(dependent_apps)} apps"
-            )
-            for app in dependent_apps:
-                print(f"  - {app.name}")
-
-        if venv.safe_to_remove():
-            venv.remove_venv()
-
-        if package_metadata.app_paths_of_dependencies:
+    if not package_metadata.apps:
+        if not package_metadata.apps_of_dependencies:
+            if venv.safe_to_remove():
+                venv.remove_venv()
             raise PipxError(
-                f"No apps associated with package {display_name}. "
-                "Try again with '--include-deps' to include apps of dependent packages, "
-                "which are listed above. "
-                "If you are attempting to install a library, pipx should not be used. "
-                "Consider using pip or a similar tool instead."
+                f"""
+                No apps associated with package {display_name} or its
+                dependencies. If you are attempting to install a library, pipx
+                should not be used. Consider using pip or a similar tool instead.
+                """
             )
-        else:
+        if package_metadata.apps_of_dependencies and not include_dependencies:
+            for (
+                dep,
+                dependent_apps,
+            ) in package_metadata.app_paths_of_dependencies.items():
+                print(
+                    f"Note: Dependent package '{dep}' contains {len(dependent_apps)} apps"
+                )
+                for app in dependent_apps:
+                    print(f"  - {app.name}")
+            if venv.safe_to_remove():
+                venv.remove_venv()
             raise PipxError(
-                f"No apps associated with package {display_name}. "
-                "If you are attempting to install a library, pipx should not be used. "
-                "Consider using pip or a similar tool instead."
+                f"""
+                No apps associated with package {display_name}. Try again
+                with '--include-deps' to include apps of dependent packages,
+                which are listed above. If you are attempting to install a
+                library, pipx should not be used. Consider using pip or a
+                similar tool instead."
+                """
             )
 
-    if package_metadata.apps:
-        pass
-    elif package_metadata.apps_of_dependencies and include_dependencies:
-        pass
-    else:
-        # No apps associated with this package and we aren't including dependencies.
-        # This package has nothing for pipx to use, so this is an error.
-        if venv.safe_to_remove():
-            venv.remove_venv()
-        raise PipxError(
-            f"No apps associated with package {display_name} or its dependencies. "
-            "If you are attempting to install a library, pipx should not be used. "
-            "Consider using pip or a similar tool instead."
-        )
-
     expose_apps_globally(
         local_bin_dir,
         package_metadata.app_paths,
@@ -328,22 +370,30 @@ def run_post_install_actions(
     if include_dependencies:
         for _, app_paths in package_metadata.app_paths_of_dependencies.items():
             expose_apps_globally(
-                local_bin_dir, app_paths, force=force, suffix=package_metadata.suffix,
+                local_bin_dir, app_paths, force=force, suffix=package_metadata.suffix
             )
 
-    print(get_package_summary(venv_dir, package=package, new_install=True,))
+    package_summary, _ = get_package_summary(
+        venv_dir, package=package, new_install=True
+    )
+    print(package_summary)
     warn_if_not_on_path(local_bin_dir)
     print(f"done! {stars}", file=sys.stderr)
 
 
-def warn_if_not_on_path(local_bin_dir: Path):
+def warn_if_not_on_path(local_bin_dir: Path) -> None:
     if not userpath.in_current_path(str(local_bin_dir)):
-        logging.warning(
-            f"{hazard}  Note: {str(local_bin_dir)!r} is not on your PATH environment "
-            "variable. These apps will not be globally accessible until "
-            "your PATH is updated. Run `pipx ensurepath` to "
-            "automatically add it, or manually modify your PATH in your shell's "
-            "config file (i.e. ~/.bashrc)."
+        logger.warning(
+            pipx_wrap(
+                f"""
+                {hazard}  Note: {str(local_bin_dir)!r} is not on your PATH
+                environment variable. These apps will not be globally
+                accessible until your PATH is updated. Run `pipx ensurepath` to
+                automatically add it, or manually modify your PATH in your
+                shell's config file (i.e. ~/.bashrc).
+                """,
+                subsequent_indent=" " * 4,
+            )
         )
 
 
