@@ -1,17 +1,17 @@
 import sys
-import textwrap
 from pathlib import Path
 from typing import List, Optional
 
 from pipx import constants
 from pipx.colors import bold
 from pipx.commands.common import package_name_from_spec, run_post_install_actions
+from pipx.constants import EXIT_CODE_INJECT_ERROR, EXIT_CODE_OK, ExitCode
 from pipx.emojies import stars
 from pipx.util import PipxError
 from pipx.venv import Venv
 
 
-def inject(
+def inject_dep(
     venv_dir: Path,
     package_name: Optional[str],
     package_spec: str,
@@ -21,25 +21,26 @@ def inject(
     include_apps: bool,
     include_dependencies: bool,
     force: bool,
-):
+) -> bool:
     if not venv_dir.exists() or not next(venv_dir.iterdir()):
         raise PipxError(
-            textwrap.dedent(
-                f"""\
-            Can't inject {package_spec!r} into nonexistent Virtual Environment {str(venv_dir)!r}.
-            Be sure to install the package first with pipx install {venv_dir.name!r}
-            before injecting into it."""
-            )
+            f"""
+            Can't inject {package_spec!r} into nonexistent Virtual Environment
+            {venv_dir.name!r}. Be sure to install the package first with 'pipx
+            install {venv_dir.name}' before injecting into it.
+            """
         )
 
     venv = Venv(venv_dir, verbose=verbose)
 
     if not venv.package_metadata:
         raise PipxError(
-            f"Can't inject {package_spec!r} into Virtual Environment {venv_dir.name!r}.\n"
-            f"    {venv_dir.name!r} has missing internal pipx metadata.\n"
-            "    It was likely installed using a pipx version before 0.15.0.0.\n"
-            f"    Please uninstall and install {venv_dir.name!r}, or reinstall-all to fix."
+            f"""
+            Can't inject {package_spec!r} into Virtual Environment
+            {venv.name!r}. {venv.name!r} has missing internal pipx metadata. It
+            was likely installed using a pipx version before 0.15.0.0. Please
+            uninstall and install {venv.name!r}, or reinstall-all to fix.
+            """
         )
 
     # package_spec is anything pip-installable, including package_name, vcs spec,
@@ -67,5 +68,41 @@ def inject(
             force=force,
         )
 
-    print(f"  injected package {bold(package_name)} into venv {bold(venv_dir.name)}")
+    print(f"  injected package {bold(package_name)} into venv {bold(venv.name)}")
     print(f"done! {stars}", file=sys.stderr)
+
+    # Any failure to install will raise PipxError, otherwise success
+    return True
+
+
+def inject(
+    venv_dir: Path,
+    package_name: Optional[str],
+    package_specs: List[str],
+    pip_args: List[str],
+    *,
+    verbose: bool,
+    include_apps: bool,
+    include_dependencies: bool,
+    force: bool,
+) -> ExitCode:
+    """Returns pipx exit code."""
+    if not include_apps and include_dependencies:
+        raise PipxError(
+            "Cannot pass --include-deps if --include-apps is not passed as well"
+        )
+    all_success = True
+    for dep in package_specs:
+        all_success &= inject_dep(
+            venv_dir,
+            None,
+            dep,
+            pip_args,
+            verbose=verbose,
+            include_apps=include_apps,
+            include_dependencies=include_dependencies,
+            force=force,
+        )
+
+    # Any failure to install will raise PipxError, otherwise success
+    return EXIT_CODE_OK if all_success else EXIT_CODE_INJECT_ERROR
