@@ -1,17 +1,27 @@
 import sys
 from pathlib import Path
-from typing import List
+from typing import List, Sequence
 
-from pipx.commands.inject import inject
+from packaging.utils import canonicalize_name
+
+import pipx.shared_libs  # import instead of from so mockable in tests
+from pipx.commands.inject import inject_dep
 from pipx.commands.install import install
 from pipx.commands.uninstall import uninstall
+from pipx.constants import EXIT_CODE_OK, EXIT_CODE_REINSTALL_VENV_NONEXISTENT, ExitCode
+from pipx.emojies import sleep
 from pipx.util import PipxError
 from pipx.venv import Venv, VenvContainer
 
 
 def reinstall(
-    *, venv_dir: Path, local_bin_dir: Path, python: str, verbose: bool,
-):
+    *, venv_dir: Path, local_bin_dir: Path, python: str, verbose: bool
+) -> ExitCode:
+    """Returns pipx exit code."""
+    if not venv_dir.exists():
+        print(f"Nothing to reinstall for {venv_dir.name} {sleep}")
+        return EXIT_CODE_REINSTALL_VENV_NONEXISTENT
+
     venv = Venv(venv_dir, verbose=verbose)
 
     if venv.pipx_metadata.main_package.package_or_url is not None:
@@ -21,6 +31,9 @@ def reinstall(
 
     uninstall(venv_dir, local_bin_dir, verbose)
 
+    # in case legacy original dir name
+    venv_dir = venv_dir.with_name(canonicalize_name(venv_dir.name))
+
     # install main package first
     install(
         venv_dir,
@@ -45,9 +58,9 @@ def reinstall(
             # This should never happen, but package_or_url is type
             #   Optional[str] so mypy thinks it could be None
             raise PipxError(
-                f"Internal Error injecting package {injected_package} into {venv_dir.name}"
+                f"Internal Error injecting package {injected_package} into {venv.name}"
             )
-        inject(
+        inject_dep(
             venv_dir,
             injected_name,
             injected_package.package_or_url,
@@ -58,6 +71,9 @@ def reinstall(
             force=True,
         )
 
+    # Any failure to install will raise PipxError, otherwise success
+    return EXIT_CODE_OK
+
 
 def reinstall_all(
     venv_container: VenvContainer,
@@ -65,14 +81,17 @@ def reinstall_all(
     python: str,
     verbose: bool,
     *,
-    skip: List[str],
-):
+    skip: Sequence[str],
+) -> ExitCode:
+    """Returns pipx exit code."""
+    pipx.shared_libs.shared_libs.upgrade(verbose=verbose)
+
     failed: List[str] = []
     for venv_dir in venv_container.iter_venv_dirs():
         if venv_dir.name in skip:
             continue
         try:
-            reinstall(
+            package_exit = reinstall(
                 venv_dir=venv_dir,
                 local_bin_dir=local_bin_dir,
                 python=python,
@@ -81,7 +100,11 @@ def reinstall_all(
         except PipxError as e:
             print(e, file=sys.stderr)
             failed.append(venv_dir.name)
+        if package_exit != 0:
+            failed.append(venv_dir.name)
     if len(failed) > 0:
         raise PipxError(
             f"The following package(s) failed to reinstall: {', '.join(failed)}"
         )
+    # Any failure to install will raise PipxError, otherwise success
+    return EXIT_CODE_OK
