@@ -1,7 +1,6 @@
 import datetime
 import hashlib
 import logging
-import subprocess
 import time
 import urllib.parse
 import urllib.request
@@ -16,12 +15,19 @@ from pipx.emojies import hazard
 from pipx.util import (
     WINDOWS,
     PipxError,
+    exec_app,
     get_pypackage_bin_path,
+    pipx_wrap,
     rmdir,
     run_pypackage_bin,
 )
 from pipx.venv import Venv
 
+logger = logging.getLogger(__name__)
+
+
+VENV_EXPIRED_FILENAME = "pipx_expired_venv"
+
 
 def run(
     app: str,
@@ -33,7 +39,7 @@ def run(
     pypackages: bool,
     verbose: bool,
     use_cache: bool,
-):
+) -> None:
     """Installs venv to temporary dir (or reuses cache), then runs app from
     package
     """
@@ -41,39 +47,46 @@ def run(
     if urllib.parse.urlparse(app).scheme:
         if not app.endswith(".py"):
             raise PipxError(
-                "pipx will only execute apps from the internet directly if "
-                "they end with '.py'. To run from an SVN, try pipx --spec URL BINARY"
+                """
+                pipx will only execute apps from the internet directly if they
+                end with '.py'. To run from an SVN, try pipx --spec URL BINARY
+                """
             )
-        logging.info("Detected url. Downloading and executing as a Python file.")
+        logger.info("Detected url. Downloading and executing as a Python file.")
 
         content = _http_get_request(app)
-        try:
-            return subprocess.run([str(python), "-c", content]).returncode
-        except KeyboardInterrupt:
-            return 1
+        # This never returns
+        exec_app([str(python), "-c", content])
 
     elif which(app):
-        logging.warning(
-            f"{hazard}  {app} is already on your PATH and installed at "
-            f"{which(app)}. Downloading and "
-            "running anyway."
+        logger.warning(
+            pipx_wrap(
+                f"""
+                {hazard}  {app} is already on your PATH and installed at
+                {which(app)}. Downloading and running anyway.
+                """,
+                subsequent_indent=" " * 4,
+            )
         )
 
     if WINDOWS and not app.endswith(".exe"):
         app = f"{app}.exe"
-        logging.info(f"Assuming app is {app!r} (Windows only)")
+        logger.info(f"Assuming app is {app!r} (Windows only)")
 
     pypackage_bin_path = get_pypackage_bin_path(app)
     if pypackage_bin_path.exists():
-        logging.info(
+        logger.info(
             f"Using app in local __pypackages__ directory at {str(pypackage_bin_path)}"
         )
-        return run_pypackage_bin(pypackage_bin_path, app_args)
+        # This never returns
+        run_pypackage_bin(pypackage_bin_path, app_args)
     if pypackages:
         raise PipxError(
-            f"'--pypackages' flag was passed, but {str(pypackage_bin_path)!r} was "
-            "not found. See https://github.com/cs01/pythonloc to learn how to "
-            "install here, or omit the flag."
+            f"""
+            '--pypackages' flag was passed, but {str(pypackage_bin_path)!r} was
+            not found. See https://github.com/cs01/pythonloc to learn how to
+            install here, or omit the flag.
+            """
         )
 
     venv_dir = _get_temporary_venv_path(package_or_url, python, pip_args, venv_args)
@@ -83,11 +96,13 @@ def run(
     _prepare_venv_cache(venv, bin_path, use_cache)
 
     if bin_path.exists():
-        logging.info(f"Reusing cached venv {venv_dir}")
-        retval = venv.run_app(app, app_args)
+        logger.info(f"Reusing cached venv {venv_dir}")
+        # This never returns
+        venv.run_app(app, app_args)
     else:
-        logging.info(f"venv location is {venv_dir}")
-        retval = _download_and_run(
+        logger.info(f"venv location is {venv_dir}")
+        # This never returns
+        _download_and_run(
             Path(venv_dir),
             package_or_url,
             app,
@@ -95,13 +110,10 @@ def run(
             python,
             pip_args,
             venv_args,
+            use_cache,
             verbose,
         )
 
-    if not use_cache:
-        rmdir(venv_dir)
-    return retval
-
 
 def _download_and_run(
     venv_dir: Path,
@@ -111,8 +123,9 @@ def _download_and_run(
     python: str,
     pip_args: List[str],
     venv_args: List[str],
+    use_cache: bool,
     verbose: bool,
-):
+) -> None:
     venv = Venv(venv_dir, python=python, verbose=verbose)
     venv.create_venv(venv_args, pip_args)
 
@@ -135,16 +148,23 @@ def _download_and_run(
     if not (venv.bin_path / app).exists():
         apps = venv.pipx_metadata.main_package.apps
         raise PipxError(
-            f"'{app}' executable script not found in package '{package_or_url}'. "
-            "Available executable scripts: "
-            f"{', '.join(b for b in apps)}"
+            f"""
+            '{app}' executable script not found in package '{package_or_url}'.
+            Available executable scripts: {', '.join(b for b in apps)}
+            """
         )
-    return venv.run_app(app, app_args)
+
+    if not use_cache:
+        # Let future _remove_all_expired_venvs know to remove this
+        (venv_dir / VENV_EXPIRED_FILENAME).touch()
+
+    # This never returns
+    venv.run_app(app, app_args)
 
 
 def _get_temporary_venv_path(
     package_or_url: str, python: str, pip_args: List[str], venv_args: List[str]
-):
+) -> Path:
     """Computes deterministic path using hashing function on arguments relevant
     to virtual environment's end state. Arguments used should result in idempotent
     virtual environment. (i.e. args passed to app aren't relevant, but args
@@ -159,33 +179,34 @@ def _get_temporary_venv_path(
     return Path(constants.PIPX_VENV_CACHEDIR) / venv_folder_name
 
 
-def _is_temporary_venv_expired(venv_dir: Path):
+def _is_temporary_venv_expired(venv_dir: Path) -> bool:
     created_time_sec = venv_dir.stat().st_ctime
     current_time_sec = time.mktime(datetime.datetime.now().timetuple())
     age = current_time_sec - created_time_sec
     expiration_threshold_sec = 60 * 60 * 24 * TEMP_VENV_EXPIRATION_THRESHOLD_DAYS
-    return age > expiration_threshold_sec
+    return age > expiration_threshold_sec or (venv_dir / VENV_EXPIRED_FILENAME).exists()
 
 
-def _prepare_venv_cache(venv: Venv, bin_path: Path, use_cache: bool):
+def _prepare_venv_cache(venv: Venv, bin_path: Path, use_cache: bool) -> None:
     venv_dir = venv.root
     if not use_cache and bin_path.exists():
-        logging.info(f"Removing cached venv {str(venv_dir)}")
+        logger.info(f"Removing cached venv {str(venv_dir)}")
         rmdir(venv_dir)
     _remove_all_expired_venvs()
 
 
-def _remove_all_expired_venvs():
+def _remove_all_expired_venvs() -> None:
     for venv_dir in Path(constants.PIPX_VENV_CACHEDIR).iterdir():
         if _is_temporary_venv_expired(venv_dir):
-            logging.info(f"Removing expired venv {str(venv_dir)}")
+            logger.info(f"Removing expired venv {str(venv_dir)}")
             rmdir(venv_dir)
 
 
-def _http_get_request(url: str):
+def _http_get_request(url: str) -> str:
     try:
         res = urllib.request.urlopen(url)
         charset = res.headers.get_content_charset() or "utf-8"  # type: ignore
         return res.read().decode(charset)
     except Exception as e:
+        logger.debug("Uncaught Exception:", exc_info=True)
         raise PipxError(str(e))
