@@ -2,11 +2,13 @@ import os
 import sys
 from pathlib import Path
 from textwrap import dedent
+from typing import NewType
 
 DEFAULT_PIPX_HOME = Path.home() / ".local/pipx"
 DEFAULT_PIPX_BIN_DIR = Path.home() / ".local/bin"
 PIPX_HOME = Path(os.environ.get("PIPX_HOME", DEFAULT_PIPX_HOME)).resolve()
 PIPX_LOCAL_VENVS = PIPX_HOME / "venvs"
+PIPX_LOG_DIR = PIPX_HOME / "logs"
 DEFAULT_PIPX_SHARED_LIBS = PIPX_HOME / "shared"
 PIPX_SHARED_LIBS = Path(
     os.environ.get("PIPX_SHARED_LIBS", DEFAULT_PIPX_SHARED_LIBS)
@@ -16,6 +18,16 @@ LOCAL_BIN_DIR = Path(os.environ.get("PIPX_BIN_DIR", DEFAULT_PIPX_BIN_DIR)).resol
 PIPX_VENV_CACHEDIR = PIPX_HOME / ".cache"
 TEMP_VENV_EXPIRATION_THRESHOLD_DAYS = 14
 
+ExitCode = NewType("ExitCode", int)
+# pipx shell exit codes
+EXIT_CODE_OK = ExitCode(0)
+EXIT_CODE_INJECT_ERROR = ExitCode(1)
+EXIT_CODE_INSTALL_VENV_EXISTS = ExitCode(1)
+EXIT_CODE_LIST_PROBLEM = ExitCode(1)
+EXIT_CODE_UNINSTALL_VENV_NONEXISTENT = ExitCode(1)
+EXIT_CODE_UNINSTALL_ERROR = ExitCode(1)
+EXIT_CODE_REINSTALL_VENV_NONEXISTENT = ExitCode(1)
+
 
 def is_windows() -> bool:
     try:
@@ -39,7 +51,7 @@ def strtobool(val: str) -> bool:
         return False
 
 
-def use_emjois():
+def use_emjois() -> bool:
     platform_emoji_support = not is_windows() and sys.getdefaultencoding() == "utf-8"
     return strtobool(str(os.getenv("USE_EMOJI", platform_emoji_support)))
 
@@ -70,7 +82,8 @@ tcsh:
     eval `register-python-argcomplete --shell tcsh pipx`
 
 fish:
-    register-python-argcomplete --shell fish pipx | source
+    # Not required to be in the config file, only run once
+    register-python-argcomplete --shell fish pipx >~/.config/fish/completions/pipx.fish
 
 """
 )
