@@ -3,24 +3,32 @@
 """The command line interface to pipx"""
 
 import argparse
-import functools
 import logging
+import logging.config
+import os
 import re
 import shlex
 import sys
 import textwrap
+import time
 import urllib.parse
+from pathlib import Path
 from typing import Dict, List
 
 import argcomplete  # type: ignore
+from packaging.utils import canonicalize_name
 
-from . import commands, constants
-from .animate import hide_cursor, show_cursor
-from .colors import bold, green
-from .interpreter import DEFAULT_PYTHON
-from .util import PipxError, mkdir
-from .venv import VenvContainer
-from .version import __version__
+from pipx import commands, constants
+from pipx.animate import hide_cursor, show_cursor
+from pipx.colors import bold, green
+from pipx.constants import ExitCode
+from pipx.emojies import hazard
+from pipx.interpreter import DEFAULT_PYTHON
+from pipx.util import PipxError, mkdir, pipx_wrap
+from pipx.venv import VenvContainer
+from pipx.version import __version__
+
+logger = logging.getLogger(__name__)
 
 
 def print_version() -> None:
@@ -28,70 +36,92 @@ def print_version() -> None:
 
 
 SPEC_HELP = textwrap.dedent(
-    """The package name or specific installation source passed to pip.
+    """\
+    The package name or specific installation source passed to pip.
     Runs `pip install -U SPEC`.
     For example `--spec mypackage==2.0.0` or `--spec  git+https://github.com/user/repo.git@branch`
     """
 )
+
 PIPX_DESCRIPTION = textwrap.dedent(
     f"""
-Install and execute apps from Python packages.
+    Install and execute apps from Python packages.
 
-Binaries can either be installed globally into isolated Virtual Environments
-or run directly in an temporary Virtual Environment.
+    Binaries can either be installed globally into isolated Virtual Environments
+    or run directly in a temporary Virtual Environment.
 
-Virtual Environment location is {str(constants.PIPX_LOCAL_VENVS)}.
-Symlinks to apps are placed in {str(constants.LOCAL_BIN_DIR)}.
+    Virtual Environment location is {str(constants.PIPX_LOCAL_VENVS)}.
+    Symlinks to apps are placed in {str(constants.LOCAL_BIN_DIR)}.
 
-Optional Environment Variables:
-PIPX_HOME: Overrides default pipx location. Virtual Environments
-will be installed to $PIPX_HOME/venvs.
-PIPX_BIN_DIR: Overrides location of app installations. Apps are symlinked
-or copied here.
-USE_EMOJI: Override emoji behavior. Default value varies based on platform.
-PIPX_DEFAULT_PYTHON: Overrides default python used for commands.
-"""
+    """
+)
+PIPX_DESCRIPTION += pipx_wrap(
+    """
+    optional environment variables:
+      PIPX_HOME             Overrides default pipx location. Virtual Environments will be installed to $PIPX_HOME/venvs.
+      PIPX_BIN_DIR          Overrides location of app installations. Apps are symlinked or copied here.
+      USE_EMOJI             Overrides emoji behavior. Default value varies based on platform.
+      PIPX_DEFAULT_PYTHON   Overrides default python used for commands.
+    """,
+    subsequent_indent=" " * 24,  # match the indent of argparse options
+    keep_newlines=True,
 )
 
-INSTALL_DESCRIPTION = f"""
-The install command is the preferred way to globally install apps
-from python packages on your system. It creates an isolated virtual
-environment for the package, then ensures the package's apps are
-accessible on your $PATH.
+DOC_DEFAULT_PYTHON = os.getenv("PIPX__DOC_DEFAULT_PYTHON", DEFAULT_PYTHON)
 
-The result: apps you can run from anywhere, located in packages
-you can cleanly upgrade or uninstall. Guaranteed to not have
-dependency version conflicts or interfere with your OS's python
-packages. 'sudo' is not required to do this.
+INSTALL_DESCRIPTION = textwrap.dedent(
+    f"""
+    The install command is the preferred way to globally install apps
+    from python packages on your system. It creates an isolated virtual
+    environment for the package, then ensures the package's apps are
+    accessible on your $PATH.
 
-pipx install PACKAGE_NAME
-pipx install --python PYTHON PACKAGE_NAME
-pipx install VCS_URL
-pipx install ./LOCAL_PATH
-pipx install ZIP_FILE
-pipx install TAR_GZ_FILE
+    The result: apps you can run from anywhere, located in packages
+    you can cleanly upgrade or uninstall. Guaranteed to not have
+    dependency version conflicts or interfere with your OS's python
+    packages. 'sudo' is not required to do this.
 
-The PACKAGE_SPEC argument is passed directly to `pip install`.
+    pipx install PACKAGE_NAME
+    pipx install --python PYTHON PACKAGE_NAME
+    pipx install VCS_URL
+    pipx install ./LOCAL_PATH
+    pipx install ZIP_FILE
+    pipx install TAR_GZ_FILE
 
-The default virtual environment location is {constants.DEFAULT_PIPX_HOME}
-and can be overridden by setting the environment variable `PIPX_HOME`
- (Virtual Environments will be installed to `$PIPX_HOME/venvs`).
+    The PACKAGE_SPEC argument is passed directly to `pip install`.
 
-The default app location is {constants.DEFAULT_PIPX_BIN_DIR} and can be
-overridden by setting the environment variable `PIPX_BIN_DIR`.
+    The default virtual environment location is {constants.DEFAULT_PIPX_HOME}
+    and can be overridden by setting the environment variable `PIPX_HOME`
+    (Virtual Environments will be installed to `$PIPX_HOME/venvs`).
 
-The default python executable used to install a package is
-{DEFAULT_PYTHON} and can be overridden by setting the environment
-variable `PIPX_DEFAULT_PYTHON`.
-"""
+    The default app location is {constants.DEFAULT_PIPX_BIN_DIR} and can be
+    overridden by setting the environment variable `PIPX_BIN_DIR`.
+
+    The default python executable used to install a package is
+    {DOC_DEFAULT_PYTHON} and can be overridden
+    by setting the environment variable `PIPX_DEFAULT_PYTHON`.
+    """
+)
 
 
 class LineWrapRawTextHelpFormatter(argparse.RawDescriptionHelpFormatter):
-    def _split_lines(self, text, width):
+    def _split_lines(self, text: str, width: int) -> List[str]:
         text = self._whitespace_matcher.sub(" ", text).strip()
         return textwrap.wrap(text, width)
 
 
+class InstalledVenvsCompleter:
+    def __init__(self, venv_container: VenvContainer) -> None:
+        self.packages = [str(p.name) for p in sorted(venv_container.iter_venv_dirs())]
+
+    def use(self, prefix: str, **kwargs) -> List[str]:
+        return [
+            f"{prefix}{x[len(prefix):]}"
+            for x in self.packages
+            if x.startswith(canonicalize_name(prefix))
+        ]
+
+
 def get_pip_args(parsed_args: Dict) -> List[str]:
     pip_args: List[str] = []
     if parsed_args.get("index_url"):
@@ -114,7 +144,7 @@ def get_venv_args(parsed_args: Dict) -> List[str]:
     return venv_args
 
 
-def run_pipx_command(args: argparse.Namespace):  # noqa: C901
+def run_pipx_command(args: argparse.Namespace) -> ExitCode:  # noqa: C901
     verbose = args.verbose if "verbose" in args else False
     pip_args = get_pip_args(vars(args))
     venv_args = get_venv_args(vars(args))
@@ -132,7 +162,9 @@ def run_pipx_command(args: argparse.Namespace):  # noqa: C901
                     args.spec = args.spec + f"#egg={package}"
 
         venv_dir = venv_container.get_venv_dir(package)
-        logging.info(f"Virtual Environment location is {venv_dir}")
+        logger.info(f"Virtual Environment location is {venv_dir}")
+    if "skip" in args:
+        skip_list = [canonicalize_name(x) for x in args.skip]
 
     if args.command == "run":
         package_or_url = (
@@ -141,7 +173,8 @@ def run_pipx_command(args: argparse.Namespace):  # noqa: C901
             else args.app_with_args[0]
         )
         use_cache = not args.no_cache
-        return commands.run(
+        # This never returns
+        commands.run(
             args.app_with_args[0],
             package_or_url,
             args.app_with_args[1:],
@@ -152,6 +185,8 @@ def run_pipx_command(args: argparse.Namespace):  # noqa: C901
             verbose,
             use_cache,
         )
+        # error, we should never reach it
+        return ExitCode(1)
     elif args.command == "install":
         return commands.install(
             None,
@@ -167,24 +202,31 @@ def run_pipx_command(args: argparse.Namespace):  # noqa: C901
             suffix=args.suffix,
         )
     elif args.command == "inject":
-        if not args.include_apps and args.include_deps:
-            raise PipxError(
-                "Cannot pass --include-deps if --include-apps is not passed as well"
-            )
-        for dep in args.dependencies:
-            commands.inject(
-                venv_dir,
-                None,
-                dep,
-                pip_args,
-                verbose=verbose,
-                include_apps=args.include_apps,
-                include_dependencies=args.include_deps,
-                force=args.force,
-            )
+        return commands.inject(
+            venv_dir,
+            None,
+            args.dependencies,
+            pip_args,
+            verbose=verbose,
+            include_apps=args.include_apps,
+            include_dependencies=args.include_deps,
+            force=args.force,
+        )
     elif args.command == "upgrade":
         return commands.upgrade(
-            venv_dir, pip_args, verbose, upgrading_all=False, force=args.force
+            venv_dir,
+            pip_args,
+            verbose,
+            include_injected=args.include_injected,
+            force=args.force,
+        )
+    elif args.command == "upgrade-all":
+        return commands.upgrade_all(
+            venv_container,
+            verbose,
+            include_injected=args.include_injected,
+            skip=skip_list,
+            force=args.force,
         )
     elif args.command == "list":
         return commands.list_packages(venv_container, args.include_injected)
@@ -192,9 +234,12 @@ def run_pipx_command(args: argparse.Namespace):  # noqa: C901
         return commands.uninstall(venv_dir, constants.LOCAL_BIN_DIR, verbose)
     elif args.command == "uninstall-all":
         return commands.uninstall_all(venv_container, constants.LOCAL_BIN_DIR, verbose)
-    elif args.command == "upgrade-all":
-        return commands.upgrade_all(
-            venv_container, verbose, skip=args.skip, force=args.force
+    elif args.command == "reinstall":
+        return commands.reinstall(
+            venv_dir=venv_dir,
+            local_bin_dir=constants.LOCAL_BIN_DIR,
+            python=args.python,
+            verbose=verbose,
         )
     elif args.command == "reinstall-all":
         return commands.reinstall_all(
@@ -202,25 +247,26 @@ def run_pipx_command(args: argparse.Namespace):  # noqa: C901
             constants.LOCAL_BIN_DIR,
             args.python,
             verbose,
-            skip=args.skip,
+            skip=skip_list,
         )
     elif args.command == "runpip":
         if not venv_dir:
-            raise PipxError("developer error: venv dir is not defined")
+            raise PipxError("Developer error: venv_dir is not defined.")
         return commands.run_pip(package, venv_dir, args.pipargs, args.verbose)
     elif args.command == "ensurepath":
         try:
             return commands.ensure_pipx_paths(force=args.force)
         except Exception as e:
-            raise PipxError(e)
+            logger.debug("Uncaught Exception:", exc_info=True)
+            raise PipxError(str(e), wrap_message=False)
     elif args.command == "completions":
         print(constants.completion_instructions)
-        return 0
+        return ExitCode(0)
     else:
         raise PipxError(f"Unknown command {args.command}")
 
 
-def add_pip_venv_args(parser):
+def add_pip_venv_args(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         "--system-site-packages",
         action="store_true",
@@ -239,19 +285,13 @@ def add_pip_venv_args(parser):
     )
 
 
-def add_include_dependencies(parser):
+def add_include_dependencies(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         "--include-deps", help="Include apps of dependent packages", action="store_true"
     )
 
 
-def _autocomplete_list_of_installed_packages(
-    venv_container: VenvContainer, *args, **kwargs
-) -> List[str]:
-    return list(str(p.name) for p in sorted(venv_container.iter_venv_dirs()))
-
-
-def _add_install(subparsers):
+def _add_install(subparsers) -> None:
     p = subparsers.add_parser(
         "install",
         help="Install a package",
@@ -286,7 +326,7 @@ def _add_install(subparsers):
     add_pip_venv_args(p)
 
 
-def _add_inject(subparsers, autocomplete_list_of_installed_packages):
+def _add_inject(subparsers, venv_completer) -> None:
     p = subparsers.add_parser(
         "inject",
         help="Install packages into an existing Virtual Environment",
@@ -295,7 +335,7 @@ def _add_inject(subparsers, autocomplete_list_of_installed_packages):
     p.add_argument(
         "package",
         help="Name of the existing pipx-managed Virtual Environment to inject into",
-    ).completer = autocomplete_list_of_installed_packages
+    ).completer = venv_completer
     p.add_argument(
         "dependencies",
         nargs="+",
@@ -317,13 +357,18 @@ def _add_inject(subparsers, autocomplete_list_of_installed_packages):
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_upgrade(subparsers, autocomplete_list_of_installed_packages):
+def _add_upgrade(subparsers, venv_completer) -> None:
     p = subparsers.add_parser(
         "upgrade",
         help="Upgrade a package",
         description="Upgrade a package in a pipx-managed Virtual Environment by running 'pip install --upgrade PACKAGE'",
     )
-    p.add_argument("package").completer = autocomplete_list_of_installed_packages
+    p.add_argument("package").completer = venv_completer
+    p.add_argument(
+        "--include-injected",
+        action="store_true",
+        help="Also upgrade packages injected into the main app's environment",
+    )
     p.add_argument(
         "--force",
         "-f",
@@ -334,13 +379,17 @@ def _add_upgrade(subparsers, autocomplete_list_of_installed_packages):
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_upgrade_all(subparsers):
+def _add_upgrade_all(subparsers) -> None:
     p = subparsers.add_parser(
         "upgrade-all",
         help="Upgrade all packages. Runs `pip install -U <pkgname>` for each package.",
         description="Upgrades all packages within their virtual environments by running 'pip install --upgrade PACKAGE'",
     )
-
+    p.add_argument(
+        "--include-injected",
+        action="store_true",
+        help="Also upgrade packages injected into the main app's environment",
+    )
     p.add_argument("--skip", nargs="+", default=[], help="skip these packages")
     p.add_argument(
         "--force",
@@ -351,17 +400,17 @@ def _add_upgrade_all(subparsers):
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_uninstall(subparsers, autocomplete_list_of_installed_packages):
+def _add_uninstall(subparsers, venv_completer) -> None:
     p = subparsers.add_parser(
         "uninstall",
         help="Uninstall a package",
         description="Uninstalls a pipx-managed Virtual Environment by deleting it and any files that point to its apps.",
     )
-    p.add_argument("package").completer = autocomplete_list_of_installed_packages
+    p.add_argument("package").completer = venv_completer
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_uninstall_all(subparsers):
+def _add_uninstall_all(subparsers) -> None:
     p = subparsers.add_parser(
         "uninstall-all",
         help="Uninstall all packages",
@@ -370,21 +419,48 @@ def _add_uninstall_all(subparsers):
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_reinstall_all(subparsers):
+def _add_reinstall(subparsers, venv_completer) -> None:
+    p = subparsers.add_parser(
+        "reinstall",
+        formatter_class=LineWrapRawTextHelpFormatter,
+        help="Reinstall a package",
+        description=textwrap.dedent(
+            """
+            Reinstalls a package.
+
+            Package is uninstalled, then installed with pipx install PACKAGE
+            with the same options used in the original install of PACKAGE.
+
+            """
+        ),
+    )
+    p.add_argument("package").completer = venv_completer
+    p.add_argument(
+        "--python",
+        default=DEFAULT_PYTHON,
+        help=(
+            "The Python executable used to recreate the Virtual Environment "
+            "and run the associated app/apps. Must be v3.5+."
+        ),
+    )
+    p.add_argument("--verbose", action="store_true")
+
+
+def _add_reinstall_all(subparsers) -> None:
     p = subparsers.add_parser(
         "reinstall-all",
         formatter_class=LineWrapRawTextHelpFormatter,
         help="Reinstall all packages",
         description=textwrap.dedent(
             """
-        Reinstalls all packages.
+            Reinstalls all packages.
 
-        Packages are uninstalled, then installed with pipx install PACKAGE
-        with the same options used in the original install of PACKAGE.
-        This is useful if you upgraded to a new version of Python and want
-        all your packages to use the latest as well.
+            Packages are uninstalled, then installed with pipx install PACKAGE
+            with the same options used in the original install of PACKAGE.
+            This is useful if you upgraded to a new version of Python and want
+            all your packages to use the latest as well.
 
-        """
+            """
         ),
     )
     p.add_argument(
@@ -399,7 +475,7 @@ def _add_reinstall_all(subparsers):
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_list(subparsers):
+def _add_list(subparsers) -> None:
     p = subparsers.add_parser(
         "list",
         help="List installed packages",
@@ -413,7 +489,7 @@ def _add_list(subparsers):
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_run(subparsers):
+def _add_run(subparsers) -> None:
     p = subparsers.add_parser(
         "run",
         formatter_class=LineWrapRawTextHelpFormatter,
@@ -424,17 +500,17 @@ def _add_run(subparsers):
         ),
         description=textwrap.dedent(
             f"""
-        Download the latest version of a package to a temporary virtual environment,
-        then run an app from it. The environment will be cached
-        and re-used for up to {constants.TEMP_VENV_EXPIRATION_THRESHOLD_DAYS} days. This
-        means subsequent calls to 'run' for the same package will be faster
-        since they can re-use the cached Virtual Environment.
+            Download the latest version of a package to a temporary virtual environment,
+            then run an app from it. The environment will be cached
+            and re-used for up to {constants.TEMP_VENV_EXPIRATION_THRESHOLD_DAYS} days. This
+            means subsequent calls to 'run' for the same package will be faster
+            since they can re-use the cached Virtual Environment.
 
-        In support of PEP 582 'run' will use apps found in a local __pypackages__
-         directory, if present. Please note that this behavior is experimental,
-         and is a acts as a companion tool to pythonloc. It may be modified or
-         removed in the future. See https://github.com/cs01/pythonloc.
-        """
+            In support of PEP 582 'run' will use apps found in a local __pypackages__
+            directory, if present. Please note that this behavior is experimental,
+            and acts as a companion tool to pythonloc. It may be modified or
+            removed in the future. See https://github.com/cs01/pythonloc.
+            """
         ),
     )
     p.add_argument(
@@ -470,7 +546,7 @@ def _add_run(subparsers):
     p.usage = re.sub(r"\.\.\.", "app ...", p.usage)
 
 
-def _add_runpip(subparsers, autocomplete_list_of_installed_packages):
+def _add_runpip(subparsers, venv_completer) -> None:
     p = subparsers.add_parser(
         "runpip",
         help="Run pip in an existing pipx-managed Virtual Environment",
@@ -479,7 +555,7 @@ def _add_runpip(subparsers, autocomplete_list_of_installed_packages):
     p.add_argument(
         "package",
         help="Name of the existing pipx-managed Virtual Environment to run pip in",
-    ).completer = autocomplete_list_of_installed_packages
+    ).completer = venv_completer
     p.add_argument(
         "pipargs",
         nargs=argparse.REMAINDER,
@@ -489,7 +565,7 @@ def _add_runpip(subparsers, autocomplete_list_of_installed_packages):
     p.add_argument("--verbose", action="store_true")
 
 
-def _add_ensurepath(subparsers):
+def _add_ensurepath(subparsers) -> None:
     p = subparsers.add_parser(
         "ensurepath",
         help=(
@@ -515,12 +591,10 @@ def _add_ensurepath(subparsers):
     )
 
 
-def get_command_parser():
+def get_command_parser() -> argparse.ArgumentParser:
     venv_container = VenvContainer(constants.PIPX_LOCAL_VENVS)
 
-    autocomplete_list_of_installed_packages = functools.partial(
-        _autocomplete_list_of_installed_packages, venv_container
-    )
+    completer_venvs = InstalledVenvsCompleter(venv_container)
 
     parser = argparse.ArgumentParser(
         formatter_class=LineWrapRawTextHelpFormatter, description=PIPX_DESCRIPTION
@@ -531,15 +605,16 @@ def get_command_parser():
     )
 
     _add_install(subparsers)
-    _add_inject(subparsers, autocomplete_list_of_installed_packages)
-    _add_upgrade(subparsers, autocomplete_list_of_installed_packages)
+    _add_inject(subparsers, completer_venvs.use)
+    _add_upgrade(subparsers, completer_venvs.use)
     _add_upgrade_all(subparsers)
-    _add_uninstall(subparsers, autocomplete_list_of_installed_packages)
+    _add_uninstall(subparsers, completer_venvs.use)
     _add_uninstall_all(subparsers)
+    _add_reinstall(subparsers, completer_venvs.use)
     _add_reinstall_all(subparsers)
     _add_list(subparsers)
     _add_run(subparsers)
-    _add_runpip(subparsers, autocomplete_list_of_installed_packages)
+    _add_runpip(subparsers, completer_venvs.use)
     _add_ensurepath(subparsers)
 
     parser.add_argument("--version", action="store_true", help="Print version and exit")
@@ -551,21 +626,83 @@ def get_command_parser():
     return parser
 
 
-def setup(args):
+def setup_log_file() -> Path:
+    max_logs = 10
+    # don't use utils.mkdir, to prevent emission of log message
+    constants.PIPX_LOG_DIR.mkdir(parents=True, exist_ok=True)
+
+    existing_logs = sorted(constants.PIPX_LOG_DIR.glob("cmd_*.log"))
+    if len(existing_logs) > max_logs:
+        for existing_log in existing_logs[:-max_logs]:
+            try:
+                existing_log.unlink()
+            except FileNotFoundError:
+                pass
+
+    datetime_str = time.strftime("%Y-%m-%d_%H.%M.%S")
+    log_file = constants.PIPX_LOG_DIR / f"cmd_{datetime_str}.log"
+    counter = 1
+    while log_file.exists() and counter < 10:
+        log_file = constants.PIPX_LOG_DIR / f"cmd_{datetime_str}_{counter}.log"
+        counter += 1
+
+    return log_file
+
+
+def setup_logging(verbose: bool) -> None:
+    pipx_str = bold(green("pipx >")) if sys.stdout.isatty() else "pipx >"
+    log_file = setup_log_file()
+
+    # "incremental" is False so previous pytest tests don't accumulate handlers
+    logging_config = {
+        "version": 1,
+        "formatters": {
+            "stream_nonverbose": {
+                "class": "logging.Formatter",
+                "format": "{message}",
+                "style": "{",
+            },
+            "stream_verbose": {
+                "class": "logging.Formatter",
+                "format": pipx_str + "({funcName}:{lineno}): {message}",
+                "style": "{",
+            },
+            "file": {
+                "class": "logging.Formatter",
+                "format": "{relativeCreated: >8.1f}ms ({funcName}:{lineno}): {message}",
+                "style": "{",
+            },
+        },
+        "handlers": {
+            "stream": {
+                "class": "logging.StreamHandler",
+                "formatter": "stream_verbose" if verbose else "stream_nonverbose",
+                "level": "INFO" if verbose else "WARNING",
+            },
+            "file": {
+                "class": "logging.FileHandler",
+                "formatter": "file",
+                "filename": str(log_file),
+                "level": "DEBUG",
+            },
+        },
+        "loggers": {"pipx": {"handlers": ["stream", "file"], "level": "DEBUG"}},
+        "incremental": False,
+    }
+    logging.config.dictConfig(logging_config)
+
+
+def setup(args: argparse.Namespace) -> None:
     if "version" in args and args.version:
         print_version()
         sys.exit(0)
 
-    if "verbose" in args and args.verbose:
-        pipx_str = bold(green("pipx >")) if sys.stdout.isatty() else "pipx >"
-        format_str = f"{pipx_str} (%(funcName)s:%(lineno)d): %(message)s"
+    setup_logging("verbose" in args and args.verbose)
 
-        logging.basicConfig(level=logging.DEBUG, format=format_str)
-    else:
-        logging.basicConfig(level=logging.WARNING, format="%(message)s")
-
-    logging.info(f"pipx version is {__version__}")
-    logging.info(f"Default python interpreter is {repr(DEFAULT_PYTHON)}")
+    logger.debug(f"{time.strftime('%Y-%m-%d %H:%M:%S')}")
+    logger.debug(f"{' '.join(sys.argv)}")
+    logger.info(f"pipx version is {__version__}")
+    logger.info(f"Default python interpreter is {repr(DEFAULT_PYTHON)}")
 
     mkdir(constants.PIPX_LOCAL_VENVS)
     mkdir(constants.LOCAL_BIN_DIR)
@@ -573,14 +710,20 @@ def setup(args):
 
     old_pipx_venv_location = constants.PIPX_LOCAL_VENVS / "pipx-app"
     if old_pipx_venv_location.exists():
-        logging.warning(
-            "A virtual environment for pipx was detected at "
-            f"{str(old_pipx_venv_location)}. The 'pipx-app' package has been renamed "
-            "back to 'pipx' (https://github.com/pipxproject/pipx/issues/82)."
+        logger.warning(
+            pipx_wrap(
+                f"""
+                {hazard}  A virtual environment for pipx was detected at
+                {str(old_pipx_venv_location)}. The 'pipx-app' package has been
+                renamed back to 'pipx'
+                (https://github.com/pipxproject/pipx/issues/82).
+                """,
+                subsequent_indent=" " * 4,
+            )
         )
 
 
-def check_args(parsed_pipx_args: argparse.Namespace):
+def check_args(parsed_pipx_args: argparse.Namespace) -> None:
     if parsed_pipx_args.command == "run":
         # we manually discard a first -- because using nargs=argparse.REMAINDER
         #   will not do it automatically
@@ -594,7 +737,7 @@ def check_args(parsed_pipx_args: argparse.Namespace):
             )
 
 
-def cli() -> int:
+def cli() -> ExitCode:
     """Entry point from command line"""
     try:
         hide_cursor()
@@ -605,13 +748,17 @@ def cli() -> int:
         check_args(parsed_pipx_args)
         if not parsed_pipx_args.command:
             parser.print_help()
-            return 1
+            return ExitCode(1)
         return run_pipx_command(parsed_pipx_args)
     except PipxError as e:
         print(str(e), file=sys.stderr)
-        return 1
+        logger.debug(f"PipxError: {e}", exc_info=True)
+        return ExitCode(1)
     except KeyboardInterrupt:
-        return 1
+        return ExitCode(1)
+    except Exception:
+        logger.debug("Uncaught Exception:", exc_info=True)
+        raise
     finally:
         show_cursor()
 
