@@ -1,10 +1,13 @@
 import json
 import logging
-import textwrap
 from pathlib import Path
-from typing import Any, Dict, List, NamedTuple, Optional
+from typing import Any, Dict, List, NamedTuple, Optional, Union
+
+from pipx.emojies import hazard
+from pipx.util import PipxError, pipx_wrap
+
+logger = logging.getLogger(__name__)
 
-from pipx.util import PipxError
 
 PIPX_INFO_FILENAME = "pipx_metadata.json"
 
@@ -17,7 +20,7 @@ class JsonEncoderHandlesPath(json.JSONEncoder):
         return super().default(obj)
 
 
-def _json_decoder_object_hook(json_dict):
+def _json_decoder_object_hook(json_dict: Dict[str, Any]) -> Union[Dict[str, Any], Path]:
     if json_dict.get("__type__", None) == "Path" and "__Path__" in json_dict:
         return Path(json_dict["__Path__"])
     return json_dict
@@ -91,9 +94,11 @@ class PipxMetadata:
             return metadata_dict
         else:
             raise PipxError(
-                f"{self.venv_dir.name}: Unknown metadata version "
-                f"{metadata_dict['pipx_metadata_version']}. "
-                "Perhaps it was installed with a later version of pipx."
+                f"""
+                {self.venv_dir.name}: Unknown metadata version
+                {metadata_dict['pipx_metadata_version']}. Perhaps it was
+                installed with a later version of pipx.
+                """
             )
 
     def from_dict(self, input_dict: Dict[str, Any]) -> None:
@@ -106,12 +111,13 @@ class PipxMetadata:
             for (name, data) in input_dict["injected_packages"].items()
         }
 
-    def _validate_before_write(self):
+    def _validate_before_write(self) -> None:
         if (
             self.main_package.package is None
             or self.main_package.package_or_url is None
             or not self.main_package.include_apps
         ):
+            logger.debug(f"PipxMetadata corrupt:\n{self.to_dict()}")
             raise PipxError("Internal Error: PipxMetadata is corrupt, cannot write.")
 
     def write(self) -> None:
@@ -126,15 +132,17 @@ class PipxMetadata:
                     cls=JsonEncoderHandlesPath,
                 )
         except IOError:
-            logging.warning(
-                textwrap.fill(
-                    f"Unable to write {PIPX_INFO_FILENAME} to {self.venv_dir}. "
-                    f"This may cause future pipx operations involving "
-                    f"{self.venv_dir.name} to fail or behave incorrectly.",
-                    width=79,
+            logger.warning(
+                pipx_wrap(
+                    f"""
+                    {hazard}  Unable to write {PIPX_INFO_FILENAME} to
+                    {self.venv_dir}.  This may cause future pipx operations
+                    involving {self.venv_dir.name} to fail or behave
+                    incorrectly.
+                    """,
+                    subsequent_indent=" " * 4,
                 )
             )
-            pass
 
     def read(self, verbose: bool = False) -> None:
         try:
@@ -144,12 +152,15 @@ class PipxMetadata:
                 )
         except IOError:  # Reset self if problem reading
             if verbose:
-                logging.warning(
-                    textwrap.fill(
-                        f"Unable to read {PIPX_INFO_FILENAME} in {self.venv_dir}. "
-                        f"This may cause this or future pipx operations involving "
-                        f"{self.venv_dir.name} to fail or behave incorrectly.",
-                        width=79,
+                logger.warning(
+                    pipx_wrap(
+                        f"""
+                        {hazard}  Unable to read {PIPX_INFO_FILENAME} in
+                        {self.venv_dir}.  This may cause this or future pipx
+                        operations involving {self.venv_dir.name} to fail or
+                        behave incorrectly.
+                        """,
+                        subsequent_indent=" " * 4,
                     )
                 )
             return
