@@ -1,25 +1,34 @@
-import logging
-from pathlib import Path
-from typing import List
-import time
 import datetime
+import logging
+import time
+from pathlib import Path
+from typing import List, Optional
 
+from pipx import constants
 from pipx.animate import animate
-from pipx.constants import PIPX_SHARED_LIBS, WINDOWS
+from pipx.constants import WINDOWS
 from pipx.interpreter import DEFAULT_PYTHON
-from pipx.util import get_site_packages, get_venv_paths, run
+from pipx.util import (
+    get_site_packages,
+    get_venv_paths,
+    run_subprocess,
+    subprocess_post_check,
+)
+
+logger = logging.getLogger(__name__)
+
 
 SHARED_LIBS_MAX_AGE_SEC = datetime.timedelta(days=30).total_seconds()
 
 
 class _SharedLibs:
-    def __init__(self):
-        self.root = PIPX_SHARED_LIBS
+    def __init__(self) -> None:
+        self.root = constants.PIPX_SHARED_LIBS
         self.bin_path, self.python_path = get_venv_paths(self.root)
         self.pip_path = self.bin_path / ("pip" if not WINDOWS else "pip.exe")
         # i.e. bin_path is ~/.local/pipx/shared/bin
         # i.e. python_path is ~/.local/pipx/shared/python
-        self._site_packages = None
+        self._site_packages: Optional[Path] = None
         self.has_been_updated_this_run = False
 
     @property
@@ -29,20 +38,24 @@ class _SharedLibs:
 
         return self._site_packages
 
-    def create(self, pip_args: List[str], verbose: bool = False):
+    def create(self, verbose: bool = False) -> None:
         if not self.is_valid:
             with animate("creating shared libraries", not verbose):
-                run([DEFAULT_PYTHON, "-m", "venv", "--clear", self.root])
+                create_process = run_subprocess(
+                    [DEFAULT_PYTHON, "-m", "venv", "--clear", self.root]
+                )
+            subprocess_post_check(create_process)
+
             # ignore installed packages to ensure no unexpected patches from the OS vendor
             # are used
-            self.upgrade(["--ignore-installed"] + pip_args, verbose)
+            self.upgrade(pip_args=["--force-reinstall"], verbose=verbose)
 
     @property
-    def is_valid(self):
+    def is_valid(self) -> bool:
         return self.python_path.is_file() and self.pip_path.is_file()
 
     @property
-    def needs_upgrade(self):
+    def needs_upgrade(self) -> bool:
         if self.has_been_updated_this_run:
             return False
 
@@ -51,18 +64,24 @@ class _SharedLibs:
 
         now = time.time()
         time_since_last_update_sec = now - self.pip_path.stat().st_mtime
-        logging.info(
-            f"Time since last upgrade of shared libs, in seconds: {time_since_last_update_sec}. "
-            f"Upgrade will be run by pipx if greater than {SHARED_LIBS_MAX_AGE_SEC}."
+        logger.info(
+            f"Time since last upgrade of shared libs, in seconds: {time_since_last_update_sec:.0f}. "
+            f"Upgrade will be run by pipx if greater than {SHARED_LIBS_MAX_AGE_SEC:.0f}."
         )
         return time_since_last_update_sec > SHARED_LIBS_MAX_AGE_SEC
 
-    def upgrade(self, pip_args: List[str], verbose: bool = False):
+    def upgrade(
+        self, *, pip_args: Optional[List[str]] = None, verbose: bool = False
+    ) -> None:
         # Don't try to upgrade multiple times per run
         if self.has_been_updated_this_run:
-            logging.info(f"Already upgraded libraries in {self.root}")
+            logger.info(f"Already upgraded libraries in {self.root}")
             return
-        logging.info(f"Upgrading shared libraries in {self.root}")
+
+        if pip_args is None:
+            pip_args = []
+
+        logger.info(f"Upgrading shared libraries in {self.root}")
 
         ignored_args = ["--editable"]
         _pip_args = [arg for arg in pip_args if arg not in ignored_args]
@@ -70,7 +89,7 @@ class _SharedLibs:
             _pip_args.append("-q")
         try:
             with animate("upgrading shared libraries", not verbose):
-                run(
+                upgrade_process = run_subprocess(
                     [
                         self.python_path,
                         "-m",
@@ -84,11 +103,13 @@ class _SharedLibs:
                         "wheel",
                     ]
                 )
-                self.has_been_updated_this_run = True
+            subprocess_post_check(upgrade_process)
+
+            self.has_been_updated_this_run = True
             self.pip_path.touch()
 
         except Exception:
-            logging.error("Failed to upgrade shared libraries", exc_info=True)
+            logger.error("Failed to upgrade shared libraries", exc_info=True)
 
 
 shared_libs = _SharedLibs()
